@@ -1,4 +1,4 @@
-/* $Id: board.cc,v 1.56 2001/09/22 14:43:02 bergo Exp $ */
+/* $Id: board.cc,v 1.59 2001/10/14 17:36:26 bergo Exp $ */
 
 /*
 
@@ -40,6 +40,8 @@ VectorPieces BareBoard::vpieces;
 
 string Board::BugTell;
 
+Board * Board::PopupOwner = 0;
+
 // --- BareBoard -----------------
 
 BareBoard::BareBoard() : WidgetProxy() {
@@ -173,6 +175,9 @@ Board::Board(bool withbugpane=false) : WidgetProxy() {
   f2=loadFallbackFont(2);
   f3=loadFallbackFont(3);
 
+  if (global.ShowCoordinates) { borx=20; bory=20; } else { borx=bory=0; }
+  morey=0;
+
   if ((!f1)||(!f2)||(!f3)) {
     cerr << "<Board::Board> ** failed to load one or more board fonts - install X11 75 dpi and 100 dpi fonts, restart X, and try again.\n";
     exit(2);
@@ -456,6 +461,8 @@ void Board::refreshPieceSet() {
     delete cur;
   cur=new PieceSet(orig);
   currently_rendered.invalidate();
+  if (global.ShowCoordinates) { borx=20; bory=20; } else { borx=bory=0; }
+  morey=0;
   queueRepaint();
 }
 
@@ -528,7 +535,7 @@ void Board::updateClock() {
 		    yidget->style->fg_gc[GTK_WIDGET_STATE (yidget)],
 		    clkbar,
 		    0, 0,
-		    sqside*8, 0,250,sqside*8);
+		    Width(), 0,250, Height());
   }
 }
 
@@ -587,7 +594,7 @@ void Board::renderClock() {
   }
 
   ef=effectiveFlip();
-  if (ef) i=8*sq-(th+10); else i=ch+10;
+  if (ef) i=Height()-(th+10); else i=ch+10;
   j=16;
   
   gdk_rgb_gc_set_foreground(gc,0xffffff);
@@ -600,7 +607,7 @@ void Board::renderClock() {
   clockString(clock.getValue(1),z);
   gdk_draw_string(clkbar,fb,gc,j,i+th,z);
 
-  if (ef) i=ch+10; else i=8*sq-(th+10);
+  if (ef) i=ch+10; else i=Height()-(th+10);
 
   gdk_rgb_gc_set_foreground(gc,0xffffff);
   if (clock.getActive()<0) {
@@ -612,11 +619,11 @@ void Board::renderClock() {
   gdk_draw_string(clkbar,fb,gc,j,i+th,z);
 
   // if we have enough space, render player names too
-  if (((sq*8 - bh*2) > 60)&&(mygame)) {
+  if (((Height() - bh*2) > 60)&&(mygame)) {
     i=ch+10+th+ch+10;
     gdk_rgb_gc_set_foreground(gc,0xffffff);
     gdk_draw_string(clkbar,fa,gc,j,i,mygame->getPlayerString(ef?0:1));
-    i=8*sq-(th+10)-ch-10;
+    i=Height()-(th+10)-ch-10;
     gdk_draw_string(clkbar,fa,gc,j,i,mygame->getPlayerString(ef?1:0));
   }
   
@@ -624,7 +631,7 @@ void Board::renderClock() {
 
   if (mygame) {
     int lr;
-    lr=sq*8-bh*2-ch*2;
+    lr=Height()-bh*2-ch*2;
     i=ch+10+th+ch+10+16+12;
     gdk_rgb_gc_set_foreground(gc,0xffffff);
     for(k=0;k<5;k++,i+=16,lr-=16) {
@@ -634,7 +641,7 @@ void Board::renderClock() {
     
     // house string is info[5]
 
-    if (strlen(info[5])) {
+    if (strlen(info[5])) {      
       if ((!global.DrawHouseStock)||(lr<96)) {
 	if (lr>=16)
 	  gdk_draw_string(clkbar,f3,gc,j,i,info[5]);
@@ -664,7 +671,7 @@ void Board::renderClock() {
 	    sprintf(z,"%d",st[si][0]);
 	    gdk_draw_string(clkbar,f3,gc,40*si+20,i+48+we,z);
 	    if (mygame->MyColor==WHITE)
-	      addTarget(new DropSource(pk[si]|WHITE,sqside*8+40*si,i+we,40,48));
+	      addTarget(new DropSource(pk[si]|WHITE,Width()+40*si,i+we,40,48));
 	  }
 	  if (st[si][1]) {
 	    Board::vpieces.drawPiece(clkbar,gc,40,40*si,i+be,pk[si]|BLACK);
@@ -672,11 +679,25 @@ void Board::renderClock() {
 	    sprintf(z,"%d",st[si][1]);
 	    gdk_draw_string(clkbar,f3,gc,40*si+20,i+48+be,z);
 	    if (mygame->MyColor==BLACK)
-	      addTarget(new DropSource(pk[si]|BLACK,sqside*8+40*si,i+be,40,48));
+	      addTarget(new DropSource(pk[si]|BLACK,Width()+40*si,i+be,40,48));
 	  }
 	}
       }
+      i+=100;
     }
+    // draw mouse if protocol menu available
+    if (global.protocol)
+      if (global.protocol->getPlayerActions()) {
+	gdk_rgb_gc_set_foreground(gc,0xffffff);
+	gdk_draw_rectangle(clkbar,gc,TRUE,170,i+8,16,18);
+	gdk_draw_line(clkbar,gc,177,i+8,177,i+4);
+	gdk_rgb_gc_set_foreground(gc,0xffcc00);
+	gdk_draw_rectangle(clkbar,gc,TRUE,180,i+8,6,8);
+	gdk_rgb_gc_set_foreground(gc,0);
+	gdk_draw_line(clkbar,gc,170,i+16,186,i+16);
+	gdk_draw_line(clkbar,gc,175,i+8,175,i+16);
+	gdk_draw_line(clkbar,gc,180,i+8,180,i+16);
+      }	
   }
 
   clock_ch=0;
@@ -758,17 +779,17 @@ void Board::update(bool sndflag=false) {
 	  for(i=0;i<LastMove.size();i++) {
 	    if (ef)
 	      ap=new FlatAnimatedPiece(LastMove[i]->Piece,
-				       sqside*(7-LastMove[i]->SX),
-				       sqside*(LastMove[i]->SY),
-				       sqside*(7-LastMove[i]->DX),
-				       sqside*(LastMove[i]->DY),
+				       borx+sqside*(7-LastMove[i]->SX),
+				       morey+bory+sqside*(LastMove[i]->SY),
+				       borx+sqside*(7-LastMove[i]->DX),
+				       morey+bory+sqside*(LastMove[i]->DY),
 				       2*LastMove[i]->distance(),sndflag);
 	    else
 	      ap=new FlatAnimatedPiece(LastMove[i]->Piece,
-				       sqside*(LastMove[i]->SX),
-				       sqside*(7-LastMove[i]->SY),
-				       sqside*(LastMove[i]->DX),
-				       sqside*(7-LastMove[i]->DY),
+				       borx+sqside*(LastMove[i]->SX),
+				       morey+bory+sqside*(7-LastMove[i]->SY),
+				       borx+sqside*(LastMove[i]->DX),
+				       morey+bory+sqside*(7-LastMove[i]->DY),
 				       2*LastMove[i]->distance(),sndflag);
 	    ap->create(yidget->window,cur,sqside);
 	    animes.push_back(ap);
@@ -923,12 +944,24 @@ void Board::walkForwardAll() {
 }
 
 void Board::outlineRectangle(GdkGC *gc, int x,int y,int color,int pen) {
-  int i,j,k;
+  int i,j,k,X,Y,S;
   gdk_rgb_gc_set_foreground(gc,color);
   j=x; k=y; if (effectiveFlip()) j=7-j; else k=7-k;
-  for(i=0;i<pen;i++)
-    gdk_draw_rectangle(yidget->window,gc,FALSE,j*sqside+i,k*sqside+i,
-		       sqside-2*i,sqside-2*i);
+  if ((global.VectorPieces)||(!cur->extruded))
+    for(i=0;i<pen;i++)
+      gdk_draw_rectangle(yidget->window,gc,FALSE,borx+j*sqside+i,
+			 morey+bory+k*sqside+i,
+			 sqside-2*i,sqside-2*i);
+  else {
+    X=borx+j*sqside; Y=morey+bory+k*sqside; S=sqside;
+    gdk_draw_rectangle(yidget->window,gc,TRUE,X,Y,pen,sqside);
+    gdk_draw_rectangle(yidget->window,gc,TRUE,X+S-pen,Y,pen,sqside);
+    gdk_draw_rectangle(yidget->window,gc,TRUE,X,Y,3*pen,pen);
+    gdk_draw_rectangle(yidget->window,gc,TRUE,X+S-3*pen,Y,3*pen,pen);
+    gdk_draw_rectangle(yidget->window,gc,TRUE,X,Y+S-pen,3*pen,pen);
+    gdk_draw_rectangle(yidget->window,gc,TRUE,X+S-3*pen,Y+S-pen,3*pen,pen);
+  }
+      
 }
 
 void Board::drawBall(GdkGC *gc, int x,int y,int color,int radius) {
@@ -936,8 +969,8 @@ void Board::drawBall(GdkGC *gc, int x,int y,int color,int radius) {
   j=x; k=y; if (effectiveFlip()) j=7-j; else k=7-k;
   gdk_rgb_gc_set_foreground(gc,color);
   gdk_draw_arc(yidget->window,gc,FALSE,
-	       j*sqside+sqside/2-radius,
-	       k*sqside+sqside/2-radius,
+	       borx+j*sqside+sqside/2-radius,
+	       morey+bory+k*sqside+sqside/2-radius,
 	       radius*2, radius*2, 0, 360*64);
 }
 
@@ -1044,6 +1077,76 @@ void Board::popupDropMenu(int col,int row,int sx,int sy,guint32 etime) {
   gtk_menu_popup(GTK_MENU(popmenu),0,0,0,0,3,etime);
 }
 
+void Board::popupProtocolMenu(int x,int y, guint32 etime) {
+  GtkWidget *mi, *hs, *popmenu;
+  vector<string *> *v;
+  char z[64];
+  int i;
+  
+  if (!global.protocol) return;
+  if (!mygame) return;
+  if (global.protocol->getPlayerActions() == NULL) return;
+
+  popmenu=gtk_menu_new();
+  PopupOwner = this;
+
+  // players
+  v=global.protocol->getPlayerActions();
+
+  for(i=0;i<v->size();i++) {
+    sprintf(z,"%s: %s",mygame->PlayerName[0], ((*v)[i])->c_str() );
+    mi=gtk_menu_item_new_with_label(z);
+    gtk_widget_show(mi);
+    gtk_menu_append(GTK_MENU(popmenu),mi);
+
+    gtk_signal_connect(GTK_OBJECT(mi),"activate",
+		       GTK_SIGNAL_FUNC(menu_whitep),
+		       (gpointer) ( (*v)[i] ) );    
+  }
+
+  mi=gtk_menu_item_new();
+  hs=gtk_hseparator_new();
+  gtk_container_add(GTK_CONTAINER(mi),hs);
+  gtk_widget_show(hs);
+  gtk_widget_show(mi);
+  gtk_menu_append(GTK_MENU(popmenu),mi);
+
+  for(i=0;i<v->size();i++) {
+    sprintf(z,"%s: %s",mygame->PlayerName[1], ((*v)[i])->c_str() );
+    mi=gtk_menu_item_new_with_label(z);
+    gtk_widget_show(mi);
+    gtk_menu_append(GTK_MENU(popmenu),mi);
+
+    gtk_signal_connect(GTK_OBJECT(mi),"activate",
+		       GTK_SIGNAL_FUNC(menu_blackp),
+		       (gpointer) ( (*v)[i] ) );
+  }
+
+  if (global.protocol->getGameActions() != 0) {
+    mi=gtk_menu_item_new();
+    hs=gtk_hseparator_new();
+    gtk_container_add(GTK_CONTAINER(mi),hs);
+    gtk_widget_show(hs);
+    gtk_widget_show(mi);
+    gtk_menu_append(GTK_MENU(popmenu),mi);
+
+    v=global.protocol->getGameActions();
+
+    for(i=0;i<v->size();i++) {
+      sprintf(z,"Game #%d: %s",mygame->GameNumber, ((*v)[i])->c_str() );
+      mi=gtk_menu_item_new_with_label(z);
+      gtk_widget_show(mi);
+      gtk_menu_append(GTK_MENU(popmenu),mi);
+      
+      gtk_signal_connect(GTK_OBJECT(mi),"activate",
+			 GTK_SIGNAL_FUNC(menu_gamep),
+			 (gpointer) ( (*v)[i] ) );
+    }
+  }
+
+  gtk_menu_popup(GTK_MENU(popmenu),0,0,0,0,3,etime);
+}
+
 void Board::dump() {
   cerr.setf(ios::hex,ios::basefield);
   cerr.setf(ios::showbase);
@@ -1077,59 +1180,94 @@ void drop_callbackQ(GtkMenuItem *item,gpointer data) {
   Board *b; b=(Board *)data; b->drop(QUEEN);
 }
 
-void Board::drawCoordinate(GdkPixmap *dest,GdkGC *gc,int side,int x,int y) {
-  bool ef;
-  char c[3];
-  int k,m;
+void menu_whitep(GtkMenuItem *item, gpointer data) {
+  Board *me;
+  me=Board::PopupOwner;
+  Board::PopupOwner = 0;
+  if (!me) return;
+  if (!me->mygame) return;
+  if (!global.protocol) return;  
+  global.protocol->callPlayerAction(me->mygame->PlayerName[0],(string *) data);
+}
 
-  c[2]=0;
-  ef=effectiveFlip();
-  
-  c[0]='a'+(ef?7-x:x);
-  c[1]='1'+(ef?y:7-y);
-  
-  gdk_rgb_gc_set_foreground(gc,0);
-  
-  for(k=-1;k<=2;k++)
-    for(m=-1;m<=1;m++)
-      if ((k)||(m))
-	gdk_draw_string(dest,f3,gc,side*x+4+k,side*y+14+m,c);
-  gdk_rgb_gc_set_foreground(gc,0xffffff);
-  gdk_draw_string(dest,f3,gc,side*x+4,side*y+14,c);
-  gdk_draw_string(dest,f3,gc,side*x+4+1,side*y+14,c);
+void menu_blackp(GtkMenuItem *item, gpointer data) {
+  Board *me;
+  me=Board::PopupOwner;
+  Board::PopupOwner = 0;
+  if (!me) return;
+  if (!me->mygame) return;
+  if (!global.protocol) return;  
+  global.protocol->callPlayerAction(me->mygame->PlayerName[1],(string *) data);
+}
+
+void menu_gamep(GtkMenuItem *item, gpointer data) {
+  Board *me;
+  me=Board::PopupOwner;
+  Board::PopupOwner = 0;
+  if (!me) return;
+  if (!me->mygame) return;
+  if (!global.protocol) return;  
+  global.protocol->callGameAction(me->mygame->GameNumber,(string *) data);
 }
 
 void Board::drawCoordinates(GdkPixmap *dest,GdkGC *gc,int side) {
-  int i,j;
-  
-  for(i=0;i<8;i++)
-    for(j=0;j<8;j++)
-      drawCoordinate(dest,gc,side,i,j);
+  bool ef;
+  int i;
+  char z[2];
+  z[1]=0;
+  ef=effectiveFlip();
+
+  if ((borx)||(bory)) {
+    gdk_rgb_gc_set_foreground(gc,0);
+    gdk_draw_rectangle(dest,gc,TRUE,0,0,Width(),bory);
+    gdk_draw_rectangle(dest,gc,TRUE,0,Height()-bory,Width(),bory);
+    gdk_draw_rectangle(dest,gc,TRUE,0,0,borx,Height());
+    gdk_draw_rectangle(dest,gc,TRUE,Width()-borx,0,borx,Height());
+  }
+
+  gdk_rgb_gc_set_foreground(gc,0x404040);
+  gdk_draw_rectangle(dest,gc,TRUE,borx,2,side*8,bory-4);
+  gdk_draw_rectangle(dest,gc,TRUE,borx,2+side*8+bory+morey,side*8,bory-4);
+  gdk_draw_rectangle(dest,gc,TRUE,2,bory+1,borx-4,morey+side*8-1);
+  gdk_draw_rectangle(dest,gc,TRUE,2+side*8+borx,bory+1,borx-4,morey+side*8-1);
+
+  gdk_rgb_gc_set_foreground(gc,0xffffff);
+
+  for(i=0;i<8;i++) {
+    z[0]=ef?'h'-i:'a'+i;
+    gdk_draw_string(dest,f3,gc, borx+side*i+side/2, bory-4, z);
+    gdk_draw_string(dest,f3,gc, borx+side*i+side/2, morey+8*side+2*bory-4, z);
+    z[0]=ef?'1'+i:'8'-i;
+    gdk_draw_string(dest,f3,gc, 6, bory+morey+side*i+side/2, z);
+    gdk_draw_string(dest,f3,gc, side*8+borx+6, bory+morey+side*i+side/2, z);
+  }
+
+
 }
 
 void Board::composeVecBoard(GdkGC *gc) {
   Position *joker;
   int i,j;
-  vpieces.drawSquares(buffer,gc,sqside);
+  vpieces.drawSquares(buffer,gc,sqside, borx, bory);
 
   if (gotAnimationLoop) joker=&fake; else joker=&position;
   if (effectiveFlip())
     for(i=0;i<8;i++)
       for(j=0;j<8;j++)
-	Board::vpieces.drawPiece(buffer,gc,sqside,i*sqside,j*sqside,
+	Board::vpieces.drawPiece(buffer,gc,sqside,borx+i*sqside,bory+j*sqside,
 				 joker->getPiece(7-i,j));
   else
     for(i=0;i<8;i++)
       for(j=0;j<8;j++)
-	Board::vpieces.drawPiece(buffer,gc,sqside,i*sqside,j*sqside,
+	Board::vpieces.drawPiece(buffer,gc,sqside,borx+i*sqside,bory+j*sqside,
 				 joker->getPiece(i,7-j));
 
   if (global.ShowCoordinates)
     drawCoordinates(buffer,gc,sqside);
 
-  i=sqside*8;
+  i=Width();
   gdk_rgb_gc_set_foreground(gc,0);
-  gdk_draw_rectangle(buffer,gc,TRUE,0,i,i,SCRAP_WIDTH-i);
+  gdk_draw_rectangle(buffer,gc,TRUE,0,i,i,SCRAP_WIDTH-Height());
 
   currently_rendered=*(joker);
 }
@@ -1139,7 +1277,7 @@ void Board::pasteVecBoard() {
 		  yidget->style->fg_gc[GTK_WIDGET_STATE (yidget)],
 		  buffer,
 		  0, 0,
-		  0, 0,sqside*8,sqside*8);
+		  0, 0,Width(),Height());
 }
 
 static GdkEventMotion LastMotionEvent;
@@ -1186,9 +1324,9 @@ gboolean board_expose_event(GtkWidget *widget,GdkEventExpose *ee,
 
   sq=me->sqside;
 
-  if (fw> (sq*8+250) )
+  if (fw> ( me->Width() + 250) )
     gdk_draw_rectangle(widget->window,widget->style->black_gc,TRUE,
-		       sq*8+250,0,fw-sq*8-250,fh);
+		       me->Width()+250,0,fw-me->Width()-250,fh);
 
   if (fh>SCRAP_WIDTH) {
     gdk_draw_rectangle(widget->window,widget->style->black_gc,TRUE,
@@ -1200,7 +1338,7 @@ gboolean board_expose_event(GtkWidget *widget,GdkEventExpose *ee,
     fw=SCRAP_WIDTH;
   
   pw=fw;
-  if (fw>sq*8) pw=sq*8;
+  if (fw> me->Width() ) pw=me->Width();
 
   // draw board+pieces
 
@@ -1223,24 +1361,26 @@ gboolean board_expose_event(GtkWidget *widget,GdkEventExpose *ee,
                   widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
                   me->clkbar,
                   0, 0,
-                  sq*8, 0,250,fh);
+                  me->Width(), 0,250,fh);
 
   // draw highlighted squares
 
   if ((me->sp)||(!me->LastMove.empty())||(me->premoveq)) {
     if (!gc)
       gc=gdk_gc_new(widget->window);
-
+    
     if ((global.HilightLastMove)&&(!me->LastMove.empty()))
       for(i=0;i<me->LastMove.size();i++) {
-	me->outlineRectangle(gc,me->LastMove[i]->SX,me->LastMove[i]->SY,0x0000ff,3);
-	me->outlineRectangle(gc,me->LastMove[i]->DX,me->LastMove[i]->DY,0x0000ff,3);
+	me->outlineRectangle(gc,me->LastMove[i]->SX,me->LastMove[i]->SY,
+			     0x0000ff,3);
+	me->outlineRectangle(gc,me->LastMove[i]->DX,me->LastMove[i]->DY,
+			     0x0000ff,3);
       }
-
+    
     if (me->sp)
       for(i=0;i<me->sp;i++)
 	me->outlineRectangle(gc,me->ax[i],me->ay[i],me->sel_color,4);
-
+    
     if (me->premoveq) {
       me->outlineRectangle(gc,me->pmx[1],me->pmy[1],0xff0000,3);
       me->drawBall(gc,me->pmx[1],me->pmy[1],0xff0000,5);
@@ -1303,10 +1443,21 @@ gboolean board_configure_event(GtkWidget *widget,GdkEventConfigure *ce,
   gdk_window_get_size(widget->window,&ww,&wh);
   ww-=200;
 
-  if (ww>wh)
-    optimal_square=wh/8;
-  else
-    optimal_square=ww/8;
+  if ( (ww-2*me->borx) > (wh-2*me->bory) ) {
+    if ( (global.VectorPieces) || (! me->orig->extruded) ) {
+      optimal_square=(wh - 2 * me->bory)/8;
+      me->morey=0;
+    } else {
+      optimal_square=(int) ((wh - 2 * me->bory)/8.50);
+      me->morey = optimal_square / 2;
+    }
+  } else {
+    optimal_square=(ww - 2 * me->borx)/8;
+    if ( (global.VectorPieces) || (! me->orig->extruded) )
+      me->morey = 0;
+    else
+      me->morey = optimal_square / 2;
+  }
 
   if (optimal_square < 10) return FALSE;
 
@@ -1341,18 +1492,25 @@ gboolean board_configure_event(GtkWidget *widget,GdkEventConfigure *ce,
     joker=&(me->position);
 
   // yes, ugly. pretty != efficient.
+
+  me->cur->beginQueueing();
+
   if (me->effectiveFlip())
     for(i=0;i<8;i++)
       for(j=0;j<8;j++)
 	me->cur->drawPieceAndSquare(joker->getPiece(7-i,j),
-				    me->scrap,i*sq,j*sq,SCRAP_WIDTH,
+				    me->scrap,me->borx+i*sq,
+				    me->morey+me->bory+j*sq,SCRAP_WIDTH,
 				    (i+j)%2);
   else
     for(i=0;i<8;i++)
       for(j=0;j<8;j++)
 	me->cur->drawPieceAndSquare(joker->getPiece(i,7-j),
-				    me->scrap,i*sq,j*sq,SCRAP_WIDTH,
+				    me->scrap,me->borx+i*sq,
+				    me->morey+me->bory+j*sq,SCRAP_WIDTH,
 				    (i+j)%2);
+
+  me->cur->endQueueing();
   // and doing without curly braces just adds to the general perplexity
 
   // copy scrap to pixmap buffer
@@ -1371,16 +1529,14 @@ gboolean board_configure_event(GtkWidget *widget,GdkEventConfigure *ce,
   return 1;
 }
 
-static bool point_in_rect(int x,int y,int x1,int y1,int w,int h) {
-  return( (x>=x1) && (x<=(x1+w)) && (y>=y1) && (y<=(y1+h)) );
+static bool not_disjoint_intervals(int a, int b, int c, int d) {
+  return( (c<=b) && (d>=a) );
 }
 
 static bool rects_overlap(int x1,int y1,int w1,int h1,
 			  int x2,int y2,int w2,int h2) {
-  return( (point_in_rect(x1,y1,x2,y2,w2,h2)||
-	   point_in_rect(x1+w1,y1,x2,y2,w2,h2)||
-	   point_in_rect(x1+w1,y1+h1,x2,y2,w2,h2)||
-	   point_in_rect(x1,y1+h1,x2,y2,w2,h2) ) );
+  return( not_disjoint_intervals(x1,x1+w1, x2, x2+w2) &&
+	  not_disjoint_intervals(y1,y1+h1, y2, y2+h2) );
 }
 
 static GdkPixmap *GCPbuffer=0;
@@ -1413,6 +1569,7 @@ static int gcpw=0,gcph=0;
    Vectorized pieces: copy square from [buffer] to [GCPbuffer],
    Bitmapped pieces:  copy square from [scrap] to [GCPbuffer].
    Both: Copy clock from [clkbar] to [GCPbuffer] if needed.
+   Extruded sets: be sure to erase enough to cover [PieceSet::height]
 
    Step 2: Erase source square
    Only if dragging from the board (instead of zhouse stock,
@@ -1421,6 +1578,8 @@ static int gcpw=0,gcph=0;
     [GCPbuffer].
    Bitmapped pieces: allocate new image (nsq), draw empty square on
     it, copy [nsq] to [GCPbuffer].
+   Extruded Bitmapped pieces: redraw piece on square right above the
+    source square.
 
    Step 3: Draw dragged piece
    Vectorized pieces: just draw it on [GCPbuffer]
@@ -1443,12 +1602,12 @@ gboolean board_motion_event(GtkWidget *widget,
 			    gpointer data) {
   Board *me;
   GdkGC *gc=0, *gcpgc;
-  rgbptr nsq=0, tsq=0;
+  rgbptr nsq=0, tsq=0, usq=0, vsq=0, ovsq=0;
   int rw,rh;
   bool clock_too=false;
   bool got_ghost=false;
   int ww,wh,sw,sh;
-  int sq;
+  int sq,he,xc,xr;
 
   memcpy(&LastMotionEvent,em,sizeof(GdkEventMotion));
 
@@ -1457,7 +1616,8 @@ gboolean board_motion_event(GtkWidget *widget,
   if ((!me->dr_fto)||(!me->dr_active)||(!em)||(!(em->state & GDK_BUTTON1_MASK)))
     return FALSE;
 
-  sq=me->sqside;
+  sq=he=me->sqside;
+  if ((me->cur->extruded)&&(!global.VectorPieces)) he=me->cur->height;
 
   // STEP 0: prepare double buffer for drawing
 
@@ -1486,10 +1646,10 @@ gboolean board_motion_event(GtkWidget *widget,
   // STEP 1: erase dirty animation square
 
   if (me->dr_step) {
-    rw=8*sq - me->dr_dirty[0];
-    rh=8*sq - me->dr_dirty[1];
+    rw= me->Width() - me->dr_dirty[0];
+    rh= me->Height() - me->dr_dirty[1];
     if (rw > sq) { rw = sq; clock_too=true; }
-    if (rh > sq) { rh = sq; clock_too=true; }
+    if (rh > he) { rh = he; clock_too=true; }
  
     if ((rw>0)&&(rh>0))
       if (global.VectorPieces) {
@@ -1515,7 +1675,7 @@ gboolean board_motion_event(GtkWidget *widget,
 		    widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
 		    me->clkbar,
 		    0, 0,
-		    sq*8, 0,250,sq*8);
+		    me->Width(), 0,250, me->Height() );
     
   }
 
@@ -1527,24 +1687,49 @@ gboolean board_motion_event(GtkWidget *widget,
       gc=gdk_gc_new(GCPbuffer);
       gdk_rgb_gc_set_foreground(gc,((me->dr_c+me->dr_r)&0x01)?global.DarkSqColor:global.LightSqColor);
       gdk_draw_rectangle(GCPbuffer,gc,TRUE,
-			 me->dr_c*sq,me->dr_r*sq,
+			 me->borx + me->dr_c*sq, 
+			 me->morey + me->bory + me->dr_r*sq,
 			 sq,sq);
       gdk_rgb_gc_set_foreground(gc,0);
       gdk_draw_rectangle(GCPbuffer,gc,FALSE,
-			 me->dr_c*sq,me->dr_r*sq,
+			 me->borx + me->dr_c*sq, 
+			 me->morey + me->bory + me->dr_r*sq,
 			 sq,sq);
-      if (global.ShowCoordinates)
-	me->drawCoordinate(GCPbuffer,gcpgc,sq,me->dr_c,me->dr_r);
     } else {
       
       // I only need *3, but glibc has some weird glitches when it
       // comes to allocating not padded to 32-bit words
-      nsq=(rgbptr)g_malloc(sq*sq*4);
+      nsq=(rgbptr)g_malloc(sq*he*4);
       me->cur->drawSquare( (me->dr_c+me->dr_r)&0x01, nsq, 0, 0, sq );
       gdk_draw_rgb_image(GCPbuffer,widget->style->black_gc,
-			 me->dr_c * sq, me->dr_r * sq,
+			 me->borx + me->dr_c * sq, 
+			 me->morey + me->bory + me->dr_r * sq,
 			 sq, sq,
-			 GDK_RGB_DITHER_NORMAL,nsq,3 * sq);      
+			 GDK_RGB_DITHER_NORMAL,nsq,3 * sq);
+
+      // erase upper portion of extruded piece
+      if (me->cur->extruded) {
+	if (me->dr_r > 0) { // there's a square above it
+	  vsq=(rgbptr)g_malloc(sq*sq*4);
+	  xc = me->effectiveFlip() ? 7 - me->dr_c : me->dr_c;
+	  xr = me->effectiveFlip() ? me->dr_r - 1 : 8 - me->dr_r ;
+	  me->cur->drawPieceAndSquare( 
+				      me->position.getPiece(xc, xr) ,
+				      nsq, 0, 0, sq, ((xc+xr)%2 == 0) );
+	  memcpy(vsq,nsq,sq*sq*3);
+
+	  gdk_draw_rgb_image(GCPbuffer,widget->style->black_gc,
+			     me->borx + me->dr_c * sq, 
+			     me->morey + me->bory + (me->dr_r - 1) * sq,
+			     sq, sq,
+			     GDK_RGB_DITHER_NORMAL,nsq,3 * sq);
+	} else { // there is nothing above it, just paint it black
+	  gdk_draw_rectangle(GCPbuffer,widget->style->black_gc,TRUE,
+			     me->borx + me->dr_c * sq,
+			     me->bory, sq, me->morey);
+	}
+      }
+
     }
   }
 
@@ -1570,42 +1755,79 @@ gboolean board_motion_event(GtkWidget *widget,
   dx = x - me->dr_ox;
   dy = y - me->dr_oy;
 
-  if (rects_overlap(dx,dy,sq,sq,me->dr_c*sq,me->dr_r*sq,sq,sq))
-    got_ghost=true; // must do tsq trick if not in vector mode
+  // yet more segfault prevention
+  if (dx < 0) { x-=dx; dx=0; }
+  if (dy < 0) { y-=dy; dy=0; }
 
   if (global.VectorPieces) {
     if (!gc)
       gc=gdk_gc_new(GCPbuffer);
     Board::vpieces.drawPiece((GdkPixmap *)(GCPbuffer),
 			     gc,sq,dx,dy,me->dr_p);    
+    xr=dy;
   } else {
     // this is gonna get tricky...
     // copy currently rendered bg to nsq...
 
     if (!nsq)
-      nsq=(rgbptr)g_malloc(sq*sq*4);
+      nsq=(rgbptr)g_malloc(sq*he*4);
+
+    if (me->cur->extruded) {
+      xr=dy-he+sq; xc=he-sq;
+      if (xr < 0) { xc+=xr; xr=0; }
+    } else { xc=0; xr=dy; }
+
+    got_ghost = rects_overlap(dx,xr,sq,sq+xc,me->borx+me->dr_c*sq,
+			      me->morey+me->bory + me->dr_r*sq,sq,sq);
 
     if (got_ghost) { // fake empty square
-      tsq=(rgbptr)g_malloc(sq*sq*4);
+      tsq=(rgbptr)g_malloc(sq*he*4);
       me->cur->bitblt(me->scrap,tsq, 
-		      me->dr_c*sq, me->dr_r*sq,SCRAP_WIDTH,
-		      0,0,sq,sq,sq);
+		      me->borx+me->dr_c*sq, 
+		      me->morey+me->bory+me->dr_r*sq-xc,SCRAP_WIDTH,
+		      0,0,sq,sq,sq+xc);
       me->cur->drawSquare( (me->dr_c+me->dr_r)&0x01, me->scrap, 
-			   me->dr_c*sq, me->dr_r*sq,
+			   me->borx+me->dr_c*sq, 
+			   me->morey+me->bory+me->dr_r*sq,
 			   SCRAP_WIDTH );
-    }
+
+      // extruded pieces
+      if ((me->cur->extruded)&&(xc>0)) {
+	usq=(rgbptr)g_malloc(sq*sq*4);
+
+	if (me->dr_r == 0) { // no sq above, paint black bg on morey region
+	  memset(nsq,0,sq*sq*3);
+	  me->cur->bitblt(me->scrap,usq,
+			  me->borx+me->dr_c*sq,me->bory,SCRAP_WIDTH,
+			  0,0,sq,sq,me->morey); // save old content
+	  me->cur->bitblt(nsq,me->scrap,
+			  0,0,sq,
+			  me->borx+me->dr_c*sq,me->bory,SCRAP_WIDTH,
+			  sq,me->morey);
+	} else {
+	  // muahahaha
+	  ovsq=(rgbptr) g_malloc(sq*sq*4);
+
+	  me->cur->bitblt(me->scrap, ovsq,
+			  me->borx+me->dr_c*sq,
+			  me->morey+me->bory+(me->dr_r-1)*sq,SCRAP_WIDTH,
+			  0,0,sq,sq,sq);
+
+	  me->cur->bitblt(vsq,me->scrap,
+			  0,0,sq,
+			  me->borx+me->dr_c*sq,
+			  me->morey+me->bory+(me->dr_r-1)*sq,SCRAP_WIDTH,
+			  sq,sq);
+	}
+
+      } // if extruded and xc > 0
+
+    } // if got_ghost
 
     me->cur->bitblt(me->scrap,nsq,
 		    dx, dy, SCRAP_WIDTH,
 		    0,0,sq,sq,sq);
 
-    if (got_ghost) { // undo the trick
-      me->cur->bitblt(tsq,me->scrap, 0,0, sq,
-		      me->dr_c*sq, me->dr_r*sq,
-		      SCRAP_WIDTH, sq, sq);
-      g_free(tsq);
-    }
-
     // draw alpha'ed piece on nsq
     me->cur->drawPiece(me->dr_p,nsq,0,0,sq);
 
@@ -1614,6 +1836,51 @@ gboolean board_motion_event(GtkWidget *widget,
 		       dx, dy,
 		       sq, sq,
 		       GDK_RGB_DITHER_NORMAL,nsq,3 * sq);
+
+    // draw upper portion (extruded only)
+    if ((me->cur->extruded)&&(xc>0)) {
+
+      // background from scrap
+      me->cur->bitblt(me->scrap,nsq,dx,xr,SCRAP_WIDTH,
+		      0,0,sq,sq,xc);
+      
+      // draw piece
+      me->cur->drawPiece(me->dr_p,nsq,0,xc,sq);
+      
+      // paste everything (piece+background) on the destination buffer
+      gdk_draw_rgb_image(GCPbuffer,widget->style->black_gc,
+			 dx, xr,
+			 sq, xc,
+			 GDK_RGB_DITHER_NORMAL,nsq,3 * sq);
+      
+    } // if extruded and xc > 0
+
+    // undo ghost effect
+    if (got_ghost) { // undo the trick
+      me->cur->bitblt(tsq,me->scrap, 0,0, sq,
+		      me->borx+me->dr_c*sq,
+		      me->morey+me->bory+me->dr_r*sq-xc,
+		      SCRAP_WIDTH, sq, sq+xc);
+
+      if ((me->dr_r == 0)&&(xc>0))
+	me->cur->bitblt(usq,me->scrap, 0,0, sq,
+			me->borx+me->dr_c*sq,me->bory,SCRAP_WIDTH,
+			sq, me->morey);
+
+      if (ovsq) {
+	  me->cur->bitblt(ovsq,me->scrap,
+			  0,0,sq,
+			  me->borx+me->dr_c*sq,
+			  me->morey+me->bory+(me->dr_r-1)*sq,SCRAP_WIDTH,
+			  sq,sq);
+	  g_free(ovsq);
+	  g_free(vsq);
+      }
+
+      if (usq) g_free(usq);
+      g_free(tsq);
+    }
+
     g_free(nsq);
   }
 
@@ -1627,7 +1894,7 @@ gboolean board_motion_event(GtkWidget *widget,
   gdk_gc_destroy(gcpgc);
 
   me->dr_dirty[0]=dx;
-  me->dr_dirty[1]=dy;
+  me->dr_dirty[1]=xr;
   me->dr_step++;
   
   if (gc)
@@ -1655,8 +1922,12 @@ gboolean board_button_release_event(GtkWidget *widget,
       drop_from_drag=true;
   }
 
-  x=((int)(be->x))/me->sqside;
-  y=((int)(be->y))/me->sqside;  
+  if ( (be->x < me->borx) || (be->y < me->morey + me->bory ) )
+    goto b_rele_nothing;
+
+  x=((int)(be->x)-me->borx)/me->sqside;
+  y=((int)(be->y)-me->bory-me->morey)/me->sqside;  
+
   if ((x>7)||(y>7)) goto b_rele_nothing;
   if (me->effectiveFlip()) x=7-x; else y=7-y;
   //
@@ -1708,14 +1979,16 @@ gboolean board_button_press_event(GtkWidget *widget,GdkEventButton *be,
   int x,y;
   me=(Board *)data;
   if (be==0) return 0;
-  if (!me->canselect) return 1;
+  if ((!me->canselect)&&(be->button!=3)) return 1;
 
-  x=((int)(be->x))/me->sqside;
-  y=((int)(be->y))/me->sqside;
+  if ( (be->x < me->borx) || (be->y < me->bory + me->morey ) ) return 1;
+
+  x=((int)(be->x)- me->borx)/me->sqside;
+  y=((int)(be->y)- me->bory - me->morey )/me->sqside;
 
   if ((x<8)&&(y<8)) {
-    me->dr_ox= ((int)(be->x)) % me->sqside;
-    me->dr_oy= ((int)(be->y)) % me->sqside;
+    me->dr_ox= ((int)(be->x) - me->borx ) % me->sqside;
+    me->dr_oy= ((int)(be->y) - me->bory - me->morey ) % me->sqside;
     me->dr_c=x;
     me->dr_r=y;
   }
@@ -1736,7 +2009,10 @@ gboolean board_button_press_event(GtkWidget *widget,GdkEventButton *be,
 	me->dr_r=0;
 	gtk_timeout_add(90,board_drag_start_timeout,
 			(gpointer)(&(me->dr_fto)));
-      }
+      }      
+    }
+    if (be->button == 3) {
+      me->popupProtocolMenu( (int) be->x, (int) be->y, be->time );
     }
     return 1;
   }
@@ -1779,7 +2055,7 @@ gboolean board_button_press_event(GtkWidget *widget,GdkEventButton *be,
       me->repaint();
     }
   }
-  if (be->button == 3) {
+  if ((be->button == 3)&&(me->canselect)) {
     me->popupDropMenu(x,y,0,0,be->time);
   }
 
@@ -2014,4 +2290,3 @@ void board_ptell(GtkWidget *b,gpointer data) {
     global.network->writeLine(z);
   }
 }
-
