@@ -1,4 +1,4 @@
-/* $Id: chess.cc,v 1.41 2001/08/05 03:33:01 bergo Exp $ */
+/* $Id: chess.cc,v 1.42 2001/10/24 11:10:21 bergo Exp $ */
 
 /*
 
@@ -83,6 +83,16 @@ ChessGame::ChessGame(int _number,int _tyme,int _inc, int _rated,
   StopClock=0;
 }
 
+ChessGame::~ChessGame() {
+  list<Position *>::iterator mi;
+
+  for(mi=moves.begin();mi!=moves.end();mi++)
+    delete(*mi);
+
+  moves.clear();
+
+}
+
 int ChessGame::operator==(int gnum) {
   return(gnum==GameNumber);
 }
@@ -297,6 +307,11 @@ void ChessGame::openMoveList() {
   }
 }
 
+void ChessGame::closeMoveList() {
+  if (mymovelist)
+    mymovelist->close();
+}
+
 void ChessGame::moveListClosed() {
   delete mymovelist;
   mymovelist=0;
