@@ -1,4 +1,4 @@
-/* $Id: chess.h,v 1.27 2001/08/05 03:33:01 bergo Exp $ */
+/* $Id: chess.h,v 1.28 2001/10/24 11:10:22 bergo Exp $ */
 
 /*
 
@@ -70,6 +70,7 @@ class ChessGame : public MoveListListener {
   ChessGame();
   ChessGame(int _number,int _tyme,int _inc, int _rated,variant _variant,
 	    char *p1,char *p2);
+  ~ChessGame();
 
   int operator==(int gnum);
 
@@ -103,6 +104,7 @@ class ChessGame : public MoveListListener {
   void goForward1();
   void goForwardAll();
   void openMoveList();
+  void closeMoveList();
 
   void setFree();
 
