@@ -1,4 +1,4 @@
-/* $Id: dlg_gamelist.cc,v 1.11 2001/05/21 23:13:41 bergo Exp $ */
+/* $Id: dlg_gamelist.cc,v 1.13 2001/10/24 11:10:22 bergo Exp $ */
 
 /*
 
@@ -421,6 +421,8 @@ void StockListDialog::trashAll() {
 	
 	if (b==global.BoardList.front())
 	  continue; // can't discard game shown in main board
+
+	cg->closeMoveList();
 	
 	global.ebook->removePage(cg->GameNumber);
 	global.removeBoard(b);
@@ -526,33 +528,36 @@ void
 stocklist_savepgn (GtkWidget * w, gpointer data)
 {
   StockListDialog *me;
+  FileDialog *fd;
+  ChessGame *cg;
+
   me=(StockListDialog *)data;
-  me->fdlg=gtk_file_selection_new("Save as PGN");
-  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (me->fdlg)->ok_button),
-		      "clicked", GTK_SIGNAL_FUNC(stocklist_svpgnok), data );  
-  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
-					 (me->fdlg)->cancel_button),
-			     "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
-			     GTK_OBJECT (me->fdlg));
-  gtk_widget_show(me->fdlg);
-  gtk_grab_add(me->fdlg);
+  fd=new FileDialog("Save as PGN");
+  
+  if (fd->run()) {
+    cg=global.getGame(me->getSelCol0());
+    if (cg)
+      cg->savePGN(fd->FileName);
+  }
+
+  delete fd;
 }
 
 void
 stocklist_loadpgn (GtkWidget * w, gpointer data)
 {
   StockListDialog *me;
-  me=(StockListDialog *)data;
+  FileDialog *fd;
 
-  me->fdlg=gtk_file_selection_new("Load PGN");
-  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (me->fdlg)->ok_button),
-		      "clicked", GTK_SIGNAL_FUNC(stocklist_ldpgnok), data );  
-  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
-					 (me->fdlg)->cancel_button),
-			     "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
-			     GTK_OBJECT (me->fdlg));
-  gtk_widget_show(me->fdlg);
-  gtk_grab_add(me->fdlg);
+  me=(StockListDialog *)data;
+  fd=new FileDialog("Load PGN");
+
+  if (fd->run()) {
+    ChessGame::LoadPGN(fd->FileName);
+    me->refresh();
+  }
+  
+  delete fd;
 }
 
 void
@@ -567,35 +572,6 @@ stocklist_editpgn (GtkWidget * w, gpointer data)
   (new PGNEditInfoDialog(cg))->show();
 }
 
-void
-stocklist_svpgnok (GtkWidget * w, gpointer data)
-{
-  StockListDialog *me;
-  ChessGame *cg;
-  me=(StockListDialog *)data;
-  char pgnfile[512];
-  strcpy(pgnfile,gtk_file_selection_get_filename (GTK_FILE_SELECTION (me->fdlg)));
-  gtk_grab_remove(me->fdlg);
-  gtk_widget_destroy(me->fdlg);
-
-  cg=global.getGame(me->getSelCol0());
-  if (cg)
-    cg->savePGN(pgnfile);
-}
-
-void
-stocklist_ldpgnok (GtkWidget * w, gpointer data)
-{
-  StockListDialog *me;
-  me=(StockListDialog *)data;
-  char pgnfile[512];
-  strcpy(pgnfile,gtk_file_selection_get_filename (GTK_FILE_SELECTION (me->fdlg)));
-  gtk_grab_remove(me->fdlg);
-  gtk_widget_destroy(me->fdlg);
-  ChessGame::LoadPGN(pgnfile);
-  me->refresh();
-}
-
 void
 stocklist_dump (GtkWidget * w, gpointer data)
 {
