@@ -2334,7 +2334,7 @@ gtk_ftext_append_entry (GtkFText * ftext, textentry * ent)
     }
 }
 
-/* the main two public functions */
+/* the public functions */
 
 void
 gtk_ftext_append (GtkFText * ftext, char *text, int len, int color)
@@ -2358,3 +2358,87 @@ gtk_ftext_append (GtkFText * ftext, char *text, int len, int color)
   gtk_signal_emit (GTK_OBJECT (ftext), ftext_signals[CHANGED]);
 }
 
+
+gint
+gtk_ftext_save_buffer(GtkFText * ftext, char *filename)
+{
+  FILE *f;
+  textentry * cur;
+
+  f=fopen(filename, "w");
+  if (!f) return FALSE;
+
+  for(cur = ftext->text_first; cur != 0; cur = cur->next) {
+    fwrite(cur->str, 1, cur->str_len, f);
+    fwrite("\n", 1, 1, f);
+  }
+
+  fclose(f);
+  return TRUE;
+}
+
+gint
+gtk_ftext_get_last_line(GtkFText * ftext)
+{
+  textentry * i;
+  gint j=0;  
+  for(i=ftext->text_first;i!=0;i=i->next) ++j;
+  return(j-1);
+}
+
+
+gint
+gtk_ftext_find_text_upwards(GtkFText * ftext, char *needle,
+			    int topline, int bottomline,
+			    gboolean hilight)
+{
+  textentry *i, *fe = 0;
+  int j;
+  char *r;
+  gint found = -1;
+  if (topline == -1) topline=0;
+  if (bottomline == -1) bottomline = gtk_ftext_get_last_line(ftext);
+
+  /* printf("search %d-%d for [%s]\n",topline,bottomline,needle); */
+
+  for(i=ftext->text_first, j=0; i!=0 ; i=i->next, j++) {
+    if (j<topline) continue;
+    if (j>bottomline) break;
+    if (strstr(i->str,needle)) {
+      found=j;
+      fe=i;
+    }
+  }
+
+  if ( (fe != 0) && (hilight) ) {
+    gtk_ftext_selection_clear(ftext);
+
+    r=strstr(fe->str, needle);
+    fe->mark_start = fe->mark_end = (r - fe->str);
+    fe->mark_end += strlen(needle);
+
+    gtk_selection_owner_set (GTK_WIDGET (ftext), GDK_SELECTION_PRIMARY,
+			     time(0));
+    gtk_ftext_selection_render (ftext, fe, fe->mark_start, fe, fe->mark_end);
+  }
+
+  return found;
+}
+
+void
+gtk_ftext_goto(GtkFText * ftext, int pseudoline)
+{
+  int address=0;
+  int j;
+  textentry *i;
+  
+  for(i=ftext->text_first, j=0; i!=0 ; i=i->next, j++) {
+    address += i->lines_taken;
+    if (j==pseudoline) break;
+  }
+
+  if (address > 0) --address;
+
+  ftext->adj->value=address;
+  gtk_signal_emit_by_name( GTK_OBJECT(ftext->adj), "value_changed" );
+}
