@@ -1,4 +1,4 @@
-/* $Id: global.cc,v 1.39 2001/09/20 22:35:23 bergo Exp $ */
+/* $Id: global.cc,v 1.40 2001/09/30 17:48:23 bergo Exp $ */
 
 /*
 
@@ -67,6 +67,8 @@ Global::Global() {
   AnimateMoves=0;
   Premove=1;
 
+  PasswordMode = 0;
+
   TabPos=0;
 
   CommLog=0;
@@ -505,6 +507,7 @@ bool Global::hasSoundFile(char *p) {
 
 void Global::setPasswordMode(int pm) {
   list<DetachedConsole *>::iterator i;
+  PasswordMode = pm;
   for(i=Consoles.begin();i!=Consoles.end();i++)
     (*i)->setPasswordMode(pm);
 }
