@@ -1,4 +1,4 @@
-/* $Id: mainwindow.cc,v 1.70 2001/09/20 22:35:23 bergo Exp $ */
+/* $Id: mainwindow.cc,v 1.72 2001/10/01 01:56:04 bergo Exp $ */
 
 /*
 
@@ -105,6 +105,10 @@ static GtkItemFactoryEntry mainwindow_mainmenu[] = {
   { "/Windows/sep3", NULL, NULL, 0, "<Separator>" },
   { "/Windows/_Detached Console",  NULL,  GTK_SIGNAL_FUNC(windows_detached), 0, NULL },
   { "/Windows/sep3", NULL, NULL, 0, "<Separator>" },
+  { "/Windows/_Find Text (upwards)...", NULL, GTK_SIGNAL_FUNC(windows_find), 0, NULL },
+  { "/Windows/Find _Previous", NULL, GTK_SIGNAL_FUNC(windows_findp), 0, NULL },
+  { "/Windows/Save _Text Buffer...", NULL, GTK_SIGNAL_FUNC(windows_savebuffer), 0, NULL },
+  { "/Windows/sep3", NULL, NULL, 0, "<Separator>" },
   { "/Windows/Save Desktop _Geometry", NULL, GTK_SIGNAL_FUNC(windows_savedesk), 0, NULL },
   { "/_Help",              NULL,         NULL,0, "<Branch>" },
   { "/Help/_Keys",NULL, GTK_SIGNAL_FUNC(help_keys), 0, NULL },
@@ -681,6 +685,10 @@ void MainWindow::historyDown() {
   gtk_entry_set_text(GTK_ENTRY(inputbox),InputHistory->moveDown());
 }
 
+void MainWindow::saveBuffer() {
+  inconsole->saveBuffer();
+}
+
 void MainWindow::saveDesk() {
   global.Desk.clear();
 
@@ -971,16 +979,27 @@ main_key_press (GtkWidget * wid, GdkEventKey * evt, gpointer data) {
   ptr=(MainWindow *)data;
 
   if (evt->state&GDK_CONTROL_MASK) {
-    if (evt->keyval == GDK_Left) {
+    switch(evt->keyval) {
+    case GDK_Left:
       ptr->gameWalk(1);
       gtk_widget_grab_focus(ptr->inputbox);
-      return 1;
-    }
-    if (evt->keyval == GDK_Right) {
+      break;
+    case GDK_Right:
       ptr->gameWalk(2);
       gtk_widget_grab_focus(ptr->inputbox);
-      return 1;
-    }    
+      break;
+    case GDK_F:
+    case GDK_f:
+      windows_find(0,0);
+      break;
+    case GDK_G:
+    case GDK_g:
+      windows_findp(0,0);
+      break;
+    default:
+      return 0;
+    }
+    return 1;
   }
 
   switch(evt->keyval) {
@@ -1145,6 +1164,21 @@ windows_savedesk(GtkWidget *w, gpointer data) {
   mainw->saveDesk();
 }
 
+void
+windows_savebuffer(GtkWidget *w, gpointer data) {
+  mainw->saveBuffer();
+}
+
+void
+windows_find(GtkWidget *w, gpointer data) {
+  mainw->inconsole->findText();
+}
+
+void
+windows_findp(GtkWidget *w, gpointer data) {
+  mainw->inconsole->findTextNext();
+}
+
 void
 help_about(gpointer data) {
   (new AboutDialog())->show();
