@@ -1,4 +1,4 @@
-/* $Id: network.cc,v 1.23 2001/06/22 04:43:21 bergo Exp $ */
+/* $Id: network.cc,v 1.24 2001/10/24 11:10:22 bergo Exp $ */
 
 /*
 
@@ -507,16 +507,24 @@ int PipeConnection::open() {
   signal(SIGPIPE,SIG_IGN);
 
   pid=fork();
+  if (pid < 0) {
+    strcpy(errorMessage,"process creation failed.");
+    return -1;
+  }
   if (!pid) {
     ::close(0); dup2(n2h[0],0);
     ::close(1); dup2(h2n[1],1);
     ::close(2); dup2(h2n[1],2);
+    ::close(n2h[1]);
+    ::close(h2n[0]);
     setpgid(getpid(),0); // to broadcast SIGKILL later
     execvp(HelperBin,arguments);
     write(1,"exec failed\n",12);
     global.debug("exec failed",HelperBin);
     _exit(2); // eek
   } else {
+    ::close(n2h[0]);
+    ::close(h2n[1]);
     memset(firstline,0,256);
     p=firstline;
     while(1)
@@ -587,7 +595,7 @@ void PipeConnection::writeLine(char *obuffer) {
     rem=strlen(obuffer);
     for(ew=0;rem;) {
       ec=write(pout,obuffer+ew,rem);
-      if (ec<0) { cerr << "oops\n"; usleep(50000); ++tol; if (tol==4) break; } else { ew+=ec; rem-=ec; }
+      if (ec<0) { usleep(50000); ++tol; if (tol==4) break; } else { ew+=ec; rem-=ec; }
     }
     write(pout,"\n",1);
     if (global.CommLog) {
