@@ -1,4 +1,4 @@
-/* $Id: pieces.cc,v 1.15 2001/06/26 03:48:14 bergo Exp $ */
+/* $Id: pieces.cc,v 1.17 2001/10/14 17:36:26 bergo Exp $ */
 
 /*
 
@@ -23,8 +23,6 @@
 
 */
 
-
-
 #include <iostream.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -33,6 +31,13 @@
 #include "pieces.h"
 #include "global.h"
 
+PReq::PReq(piece a, rgbptr b, int c, int d, int e, bool f) {
+  p=a; dest=b; x=c; y=d; dwidth=e; onlytop=f;
+}
+
+int PReq::operator< (PReq * a) { return(y < a->y); }
+int PReq::operator> (PReq * a) { return(y > a->y); }
+
 //
 // Hint about gdk_imlib_changed_image() :
 //
@@ -72,8 +77,22 @@ PieceSet::PieceSet(char *name, char *sqname) {
       loadSquares(wherever);
   }
 
+  side = height = isrc->rgb_width / 7;
+  bottom_only=false;
+  top_only=false;
+  queuef=false;
+
+  i = (isrc->rgb_width  * 10) / isrc->rgb_height;
+
+  if ((i<34)||(i>36)) {
+    extruded=true;
+    height=isrc->rgb_height / 2;
+  } else {
+    extruded=false;
+  }
+
   initCache();
-  side=isrc->rgb_width / 7;
+
   bgcolor =((int)(isrc->rgb_data[0]))<<16;
   bgcolor|=((int)(isrc->rgb_data[1]))<<8;
   bgcolor|=((int)(isrc->rgb_data[2]));
@@ -89,18 +108,31 @@ PieceSet::PieceSet(PieceSet *src) {
   isrc=gdk_imlib_clone_image(src->isrc);
   gdk_imlib_changed_image(isrc);
   side=src->side;
+  height=src->height;
+  extruded=src->extruded;
+  queuef=src->queuef;
   bgcolor=src->bgcolor;
   alphamask=(rgbptr)g_malloc(side*side*14);
   memcpy(alphamask,src->alphamask,side*side*14);
+  bottom_only=false;
+  top_only=false;
   init_cheap();
 }
 
 PieceSet::~PieceSet() {
+  list<PReq *>::iterator qi;
   if (alphamask)
     g_free(alphamask);
   gdk_imlib_destroy_image(isrc);
   del_cheap();
   clearCache();
+
+  if (!myq.empty()) {
+    for(qi=myq.begin();qi!=myq.end();qi++)
+      delete(*qi);
+    myq.clear();
+  }
+
 }
 
 char * PieceSet::getName() {
@@ -114,7 +146,7 @@ char * PieceSet::getSquareName() {
 void PieceSet::loadSquares(char *path) {
   GdkImlibImage *tmp,*aux;
   int w1,w2,h1,h2;
-  int x,y,w,h;
+  int x,w;
 
   global.debug("PieceSet","loadSquares",path);
 
@@ -131,17 +163,43 @@ void PieceSet::loadSquares(char *path) {
   h1=isrc->rgb_height;
   h2=tmp->rgb_height;
 
-  if ( (w1!=w2) || ( h1!=h2) ) {
-    aux=gdk_imlib_clone_scaled_image(tmp,w1,h1);
+  w=w2/7;
+
+  bitblt(tmp->rgb_data, tmp->rgb_data,
+	 6*w, 0, w2,
+	 0, 0, w2,
+	 w, w);
+
+  bitblt(tmp->rgb_data, tmp->rgb_data,
+	 6*w, h2/2, w2,
+	 0, w, w2,
+	 w, w);
+
+  gdk_imlib_crop_image(tmp,0,0,w, 2*w);
+
+  if ( (w1!=w2) ) {
+    w=w1/7;
+    aux=gdk_imlib_clone_scaled_image(tmp,w,2*w);
     gdk_imlib_destroy_image(tmp);
     tmp=aux;
     gdk_imlib_changed_image(tmp);
   }
 
-  x=(w1/7)*6; y=0;
-  w=(w1/7); h=h1;
+  x=(w1/7)*6;
+  w=(w1/7);
+
+  // white square
+  bitblt(tmp->rgb_data,isrc->rgb_data,
+	 0,0,w,
+	 x,0,w1, 
+	 w,w);
+
+  // black square
+  bitblt(tmp->rgb_data,isrc->rgb_data,
+	 0, w, w,
+	 x, isrc->rgb_height / 2, w1, 
+	 w,w);
 
-  bitblt(tmp->rgb_data,isrc->rgb_data,x,y,w1,x,y,w1,w,h);
   gdk_imlib_destroy_image(tmp);
 }
 
@@ -186,6 +244,7 @@ void PieceSet::buildCache() {
     memcpy(xxcache[i+12],xxcache[12],itemlen);
   }
 
+  bottom_only=true;
   for(i=0;i<24;i+=12) {
     drawPiece(WHITE|PAWN,  xxcache[i],0,0,side);
     drawPiece(WHITE|ROOK,  xxcache[i+1],0,0,side);
@@ -200,6 +259,7 @@ void PieceSet::buildCache() {
     drawPiece(BLACK|QUEEN, xxcache[i+10],0,0,side);
     drawPiece(BLACK|KING,  xxcache[i+11],0,0,side);
   }
+  bottom_only=false;
 }
 
 void PieceSet::clearCache() {
@@ -212,11 +272,17 @@ void PieceSet::clearCache() {
 void PieceSet::scale(int newside) {
   GdkImlibImage *ni;
   float factor;
+  int newheight;
 
   //  cerr << "scaling from " << side << " to " << newside << endl;
 
   factor = ((float)newside) / ((float)side);
 
+  if (extruded)
+    newheight = (int) (factor * height);
+  else
+    newheight = newside;
+
   if (factor <= 0.2500) {
     prescale(1,4);
     goto no_more_prescale;
@@ -232,9 +298,10 @@ void PieceSet::scale(int newside) {
 
   no_more_prescale:
 
-  ni=gdk_imlib_clone_scaled_image(isrc,newside*7,newside*2);
+  ni=gdk_imlib_clone_scaled_image(isrc,newside*7,newheight*2);
   gdk_imlib_changed_image(ni);
   side=newside;
+  height=newheight;
   gdk_imlib_destroy_image(isrc);
   isrc=ni;
   calcAlphaMask();
@@ -255,7 +322,7 @@ void PieceSet::calcAlphaMask() {
   if (alphamask!=0)
     g_free(alphamask);
 
-  c=side*side*14;
+  c=side*height*14;
 
   alphamask=(rgbptr)g_malloc(c);
   memset(alphamask,0,c);
@@ -280,13 +347,37 @@ void PieceSet::calcAlphaMask() {
 
 void PieceSet::drawPiece(piece p,rgbptr dest,int x,int y,int dwidth) {
   int sx=0, sy=0;
+  int top, sc=0, hlt=0;
   p&=CUTFLAGS;
   if (p==EMPTY) return;
-  if (p&BLACK)  sy=side;
+
+  if (queuef) {
+    queueRequest(new PReq(p,dest,x,y,dwidth,false));
+    return;
+  }
+
+  if (p&BLACK)  sy=height;
   sx=side*((p&PIECE_MASK)-1);
-  bitblt_a(isrc->rgb_data,dest,sx,sy,7*side,x,y,dwidth,side,side);
+
+  if (extruded) {
+    if (bottom_only) {
+      sc=height-side; top=y;
+    } else {
+      top=y-(height-side);
+      if (top < 0) { sc = -top; top=0; }
+    }
+
+    if (top_only) hlt=height-side;
+    if (sc<height)
+      bitblt_a(isrc->rgb_data,dest,
+	       sx,sy+sc,7*side,
+	       x,top,dwidth,
+	       side,height-sc-hlt);
+  } else
+    bitblt_a(isrc->rgb_data,dest,sx,sy,7*side,x,y,dwidth,side,side);    
 }
 
+
 void PieceSet::drawSquare(int dark,rgbptr dest,int x,int y,int dwidth) {
   int sx,sy=0;
 
@@ -306,7 +397,7 @@ void PieceSet::drawSquare(int dark,rgbptr dest,int x,int y,int dwidth) {
       sq+=lo;
     }    
   } else {
-    if (dark) sy=side;
+    if (dark) sy=height;
     sx=6*side;  
     bitblt(isrc->rgb_data,dest,sx,sy,7*side,x,y,dwidth,side,side);
   }
@@ -334,6 +425,9 @@ void PieceSet::drawPieceAndSquare(piece p,rgbptr dest,
     buildCache();
   i=cacheindex[p]+(sqdark?12:0);
   bitblt(xxcache[i],dest,0,0,side,x,y,dwidth,side,side);
+
+  if (queuef)
+    queueRequest(new PReq(p,dest,x,y,dwidth,true));
 }
 
 void PieceSet::bitblt(rgbptr src,rgbptr dest,int sx,int sy,int sw,
@@ -700,6 +794,35 @@ void PieceSet::antialias_c() {
   g_free(imgclone);
 }
 
+void PieceSet::queueRequest(PReq *r) {
+  list<PReq *>::iterator qi, qj;
+
+  if (myq.empty()) { myq.push_back(r); return; }
+
+  for(qi=myq.begin();qi!=myq.end();qi=qj) {
+    qj=qi; ++qj;
+    if (qj==myq.end())  { myq.push_back(r); return; }
+    if ( (*r) < (*qj) ) { myq.insert(qj,r); return; }
+  }
+}
+
+void PieceSet::beginQueueing() {
+  queuef=extruded;
+}
+
+void PieceSet::endQueueing() {
+  list<PReq *>::iterator qi;
+  if (!queuef) return;
+  queuef=false;
+  for(qi=myq.begin();qi!=myq.end();qi++) {
+    top_only=(*qi)->onlytop;
+    drawPiece((*qi)->p, (*qi)->dest, (*qi)->x, (*qi)->y, (*qi)->dwidth);
+    delete(*qi);
+  }
+  myq.clear();
+  top_only=false;
+}
+
 // ------------------------------ vectorized pieces
 
 // offsets 32 67 90 76 65 46 
@@ -770,17 +893,17 @@ VectorPieces::VectorPieces() {
   rescale(108);
 }
 
-void VectorPieces::drawSquares(GdkPixmap *dest,GdkGC *gc,int sqside) {
+void VectorPieces::drawSquares(GdkPixmap *dest,GdkGC *gc,int sqside, int dx=0, int dy=0) {
   int i,j;
   for(i=0;i<8;i++)
     for(j=0;j<8;j++) {      
       gdk_rgb_gc_set_foreground(gc,((j+i)&0x01)?global.DarkSqColor:global.LightSqColor);
-      gdk_draw_rectangle(dest,gc,TRUE,i*sqside,j*sqside,sqside,sqside);
+      gdk_draw_rectangle(dest,gc,TRUE,dx+i*sqside,dy+j*sqside,sqside,sqside);
     }
   gdk_rgb_gc_set_foreground(gc,0);
   for(i=0;i<8;i++) {
-    gdk_draw_line(dest,gc,0,i*sqside,8*sqside,i*sqside);
-    gdk_draw_line(dest,gc,i*sqside,0,i*sqside,8*sqside);
+    gdk_draw_line(dest,gc,dx,dy+i*sqside,dx+8*sqside,dy+i*sqside);
+    gdk_draw_line(dest,gc,dx+i*sqside,dy,dx+i*sqside,dy+8*sqside);
   }
 }
 
