@@ -1,4 +1,4 @@
-/* $Id: proto_fics.cc,v 1.61 2001/09/06 03:35:28 bergo Exp $ */
+/* $Id: proto_fics.cc,v 1.62 2001/10/13 00:22:10 bergo Exp $ */
 
 /*
 
@@ -40,6 +40,15 @@ FicsProtocolParser::FicsProtocolParser() {
 
   global.debug("FicsProtocolParser","FicsProtocolParser");
 
+  GActions.push_back(new string("All Observers"));
+  GActions.push_back(new string("Game Info"));
+
+  PActions.push_back(new string("Finger"));
+  PActions.push_back(new string("Stats"));
+  PActions.push_back(new string("History"));
+  PActions.push_back(new string("Ping"));
+  PActions.push_back(new string("Log"));
+
   AuthenticationGone=0;
   LastWasPrompt=0;
   LastWasCTell=-1;
@@ -1456,3 +1465,59 @@ void FicsProtocolParser::seekRemove(ExtPatternMatcher &pm) {
   while(p=t.token(" \t\n"))
     global.skgraph->remove(atoi(p));
 }
+
+vector<string *> * FicsProtocolParser::getPlayerActions() {
+  return(&PActions);
+}
+
+vector<string *> * FicsProtocolParser::getGameActions() {
+  return(&GActions);
+}
+
+void FicsProtocolParser::callPlayerAction(char *player, string *action) {
+  for(int i=0;i<PActions.size();i++)
+    if ( (*PActions[i]) == (*action) ) { doPlayerAction(player, i); break; }
+}
+
+void FicsProtocolParser::callGameAction(int gameid, string *action) {
+  for(int i=0;i<GActions.size();i++)
+    if ( (*GActions[i]) == (*action) ) { doGameAction(gameid, i); break; }
+}
+
+void FicsProtocolParser::doPlayerAction(char *player, int id) {
+  char z[256],w[256];
+  bool ok=true;
+
+  switch(id) {
+  case 0: sprintf(z,"finger %s",player); break;
+  case 1: sprintf(z,"stat %s",player); break;
+  case 2: sprintf(z,"history %s",player); break;
+  case 3: sprintf(z,"ping %s",player); break;
+  case 4: sprintf(z,"log %s",player); break;
+  default: ok=false;
+  }
+
+  if (ok) {
+    sprintf(w,"> [issued from menu] %s",z);
+    global.output->append(w,global.SelfInputColor);
+    global.network->writeLine(z);
+  }
+}
+
+void FicsProtocolParser::doGameAction(int gameid, int id) {
+  char z[256], w[256];
+  bool ok=true;
+
+  switch(id) {
+  case 0: sprintf(z,"allob %d",gameid); break;
+  case 1: sprintf(z,"ginfo %d",gameid); break;
+  default: ok=false;
+  }
+
+  if (ok) {
+    sprintf(w,"> [issued from menu] %s",z);
+    global.output->append(w,global.SelfInputColor);
+    global.network->writeLine(z);
+  }
+}
+
