@@ -1,4 +1,4 @@
-/* $Id: proto_fics.h,v 1.30 2001/09/04 19:27:19 bergo Exp $ */
+/* $Id: proto_fics.h,v 1.31 2001/10/13 00:22:10 bergo Exp $ */
 
 /*
 
@@ -57,6 +57,12 @@ class FicsProtocolParser : public ProtocolParser {
   void exaBackward(int n);
 
   void refreshSeeks();
+
+  vector<string *> * getPlayerActions();
+  vector<string *> * getGameActions();
+
+  void callPlayerAction(char *player, string *action);
+  void callGameAction(int gameid, string *action);
  
  private:
   void createGame(ExtPatternMatcher &pm);
@@ -86,6 +92,9 @@ class FicsProtocolParser : public ProtocolParser {
   void seekAdd(ExtPatternMatcher &pm);
   void seekRemove(ExtPatternMatcher &pm);
 
+  void doPlayerAction(char *player, int id);
+  void doGameAction(int gameid, int id);
+
   int AuthenticationGone;
 
   PatternBinder Binder;
@@ -166,6 +175,7 @@ class FicsProtocolParser : public ProtocolParser {
 
   list<Position *> MoveBuf;
   deque<int> MoveRetrievals;
+  vector<string *> PActions, GActions;
 
   GameListConsumer *lastGLC, *lastALC;
 
