@@ -1,4 +1,4 @@
-/* $Id: sound.cc,v 1.5 2001/06/26 02:08:19 bergo Exp $ */
+/* $Id: sound.cc,v 1.6 2001/09/30 17:48:23 bergo Exp $ */
 
 /*
 
@@ -446,27 +446,15 @@ void snddlg_picktheme(GtkMenuItem *w,gpointer data) {
 
 void snddlg_browse(GtkWidget *w,gpointer data) {
   SoundEventDialog *me;
-  me=(SoundEventDialog *)data;
+  FileDialog *fd;
 
-  GtkWidget *dlg;
-  dlg=gtk_file_selection_new("Browse");
-  me->fdlg=dlg;
-  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (dlg)->ok_button),
-		      "clicked", GTK_SIGNAL_FUNC(snddlg_brwok), data );  
-  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
-					 (dlg)->cancel_button),
-			     "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
-			     GTK_OBJECT (dlg));
-  gtk_widget_show(dlg);
-  gtk_grab_add(dlg);
-}
-
-void snddlg_brwok(GtkWidget *w,gpointer data) {
-  SoundEventDialog *me;
   me=(SoundEventDialog *)data;
-  gtk_entry_set_text(GTK_ENTRY(me->en[3]),gtk_file_selection_get_filename (GTK_FILE_SELECTION (me->fdlg)));
-  gtk_grab_remove(me->fdlg);
-  gtk_widget_destroy(me->fdlg);
+  fd=new FileDialog("Browse");
+  
+  if (fd->run()) {
+    gtk_entry_set_text(GTK_ENTRY(me->en[3]), fd->FileName);
+  }
+  delete fd;
 }
 
 void snddlg_ok(GtkWidget *w,gpointer data) {
