@@ -1,4 +1,4 @@
-/* $Id: widgetproxy.h,v 1.9 2001/09/09 17:40:49 bergo Exp $ */
+/* $Id: widgetproxy.h,v 1.11 2001/10/01 01:56:04 bergo Exp $ */
 
 /*
 
@@ -61,6 +61,7 @@ class ModalDialog : public WidgetProxy {
  protected:
   void setDismiss(GtkObject *obj,char *sig);
   void release();
+  void releaseWithoutDelete();
 
   GtkWidget *focused_widget;
 
@@ -117,4 +118,23 @@ class TextPreview : public WidgetProxy, public UpdateNotify {
 				 gpointer data);
 };
 
+class FileDialog : public WidgetProxy {
+
+ public:
+  FileDialog(char *title);
+  ~FileDialog();
+
+  bool run();
+
+  char FileName[128];
+
+ private:
+  bool gotresult;
+  bool destroyed;
+
+  friend void filedlg_ok (GtkWidget * w, gpointer data);
+  friend void filedlg_destroy (GtkWidget * w, gpointer data);
+
+};
+
 #endif
